// ------------------------------------------------------------------------------------------------
// -----                                       GPIO_PORT                                      -----
// ------------------------------------------------------------------------------------------------

typedef struct {                            /*!< (@ 0x50000000) GPIO_PORT Structure    */
  union {
    struct {
      __IO uint8_t B0[32];                  /*!< (@ 0x50000000) Byte pin registers port 0; pins PIO0_0 to PIO0_31 */
      __IO uint8_t B1[32];                  /*!< (@ 0x50000020) Byte pin registers port 1 */
    };
    __IO uint8_t B[64];                     /*!< (@ 0x50000000) Byte pin registers port 0/1 */
  };
  __I  uint32_t RESERVED0[1008];
  union {
    struct {
      __IO uint32_t W0[32];                 /*!< (@ 0x50001000) Word pin registers port 0 */
      __IO uint32_t W1[32];                 /*!< (@ 0x50001080) Word pin registers port 1 */
    };
    __IO uint32_t W[64];                    /*!< (@ 0x50001000) Word pin registers port 0/1 */
  };
  __I  uint32_t RESERVED1[960];
  __IO uint32_t DIR[2];                     /*!< (@ 0x50002000) Direction registers port 0/1 */
  __I  uint32_t RESERVED2[30];
  __IO uint32_t MASK[2];                    /*!< (@ 0x50002080) Mask register port 0/1 */
  __I  uint32_t RESERVED3[30];
  __IO uint32_t PIN[2];                     /*!< (@ 0x50002100) Portpin register port 0 */
  __I  uint32_t RESERVED4[30];
  __IO uint32_t MPIN[2];                    /*!< (@ 0x50002180) Masked port register port 0/1 */
  __I  uint32_t RESERVED5[30];
  __IO uint32_t SET[2];                     /*!< (@ 0x50002200) Write: Set register for port 0/1 Read: output bits for port 0/1 */
  __I  uint32_t RESERVED6[30];
  __O  uint32_t CLR[2];                     /*!< (@ 0x50002280) Clear port 0/1         */
  __I  uint32_t RESERVED7[30];
  __O  uint32_t NOT[2];                     /*!< (@ 0x50002300) Toggle port 0/1        */
} LPC_GPIO_Type;


/*****************************************************************************
** Function name:		GPIOGetPinValue
**
** Descriptions:		Read Current state of port pin, PIN register value
**
** parameters:			port num, bit position
** Returned value:		None
**
*****************************************************************************/
uint32_t GPIOGetPinValue( uint32_t portNum, uint32_t bitPosi )
{
  uint32_t regVal = 0;	

  if( bitPosi < 0x20 )
  {	
	if ( LPC_GPIO->PIN[portNum] & (0x1<<bitPosi) )
	{
	  regVal = 1;
	}
  }
  else if( bitPosi == 0xFF )
  {
	regVal = LPC_GPIO->PIN[portNum];
  }
  return ( regVal );		
}

/*****************************************************************************
** Function name:		GPIOSetBitValue
**
** Descriptions:		Set/clear a bit in a specific position
**
** parameters:			port num, bit position, bit value
** 						
** Returned value:		None
**
*****************************************************************************/
void GPIOSetBitValue( uint32_t portNum, uint32_t bitPosi, uint32_t bitVal )
{
  if ( bitVal )
  {
	LPC_GPIO->SET[portNum] = 1<<bitPosi;
  }
  else
  {
	LPC_GPIO->CLR[portNum] = 1<<bitPosi;
  }
  return;
}

/*****************************************************************************
** Function name:		GPIOSetDir
**
** Descriptions:		Set the direction in GPIO port
**
** parameters:			portNum, bit position, direction (1 out, 0 input)
** 						
** Returned value:		None
**
*****************************************************************************/
void GPIOSetDir( uint32_t portNum, uint32_t bitPosi, uint32_t dir )
{
  if( dir )
  {
	LPC_GPIO->DIR[portNum] |= (1<<bitPosi);
  }
  else
  {
	LPC_GPIO->DIR[portNum] &= ~(1<<bitPosi);
  }
  return;
}

GPIO::GPIO(int pin)
{
	LPC_GPIO0->DIR |= 1<<pin;
}

void GPIO::GPIO_write(char port, char bit,  char value)
{
	// Take advantage of the fact the GPIO registers are bit-banded
	(*((uint32_t *) ((LPC_GPIO0_BASE + (port << 16)) + ((1 << bit) << 2)))) = value ? 0xFFF : 0;
}



LPC_GPIO1->FIODIR |= 1 << 29; // Put Pin connected to LED to ouput mode
LPC_GPIO1->FIOPIN ^= 1 << 29; // toggle LED

